@@ -2,8 +2,8 @@ use crate::proto::gen::tasks::Variable;
 use crate::spec::chart::{ChartSpec, MutChartVisitor};
 use crate::spec::data::DataSpec;
 use crate::spec::scale::{
-    ScaleDataReferenceSort, ScaleDataReferenceSortParameters, ScaleDataReferenceSpec,
-    ScaleDataReferencesSpec, ScaleDomainSpec, ScaleSpec, ScaleTypeSpec,
+    ScaleDataReferenceOrSignalSpec, ScaleDataReferenceSort, ScaleDataReferenceSortParameters,
+    ScaleDomainSpec, ScaleFieldReferenceSpec, ScaleFieldsReferencesSpec, ScaleSpec, ScaleTypeSpec,
 };
 use crate::spec::transform::aggregate::AggregateOpSpec;
 use crate::task_graph::graph::ScopedVariable;
@@ -59,7 +59,7 @@ impl<'a> MutChartVisitor for SplitScaleDomainVisitor<'a> {
                 ScaleDomainSpec::FieldReference(field_ref) => {
                     self.split_field_reference_domain(scale, scope, &field_ref)?;
                 }
-                ScaleDomainSpec::FieldsReference(fields_ref) => {
+                ScaleDomainSpec::FieldsReferences(fields_ref) => {
                     self.split_fields_reference_domain(scale, scope, &fields_ref)?;
                 }
                 _ => {}
@@ -74,7 +74,7 @@ impl<'a> SplitScaleDomainVisitor<'a> {
         &mut self,
         scale: &mut ScaleSpec,
         scope: &[u32],
-        fields_ref: &ScaleDataReferencesSpec,
+        fields_ref: &ScaleFieldsReferencesSpec,
     ) -> Result<()> {
         let discrete_scale = scale.type_.clone().unwrap_or_default().is_discrete();
         let (new_datasets, new_dataset_scope, new_domain) = if discrete_scale {
@@ -91,34 +91,36 @@ impl<'a> SplitScaleDomainVisitor<'a> {
             let mut new_dataset_scope = Vec::new();
             let mut new_fields = Vec::new();
             for (field_index, field_ref) in fields_ref.fields.iter().enumerate() {
-                let field_name = &field_ref.field;
-                let data_name = field_ref.data.clone();
-                let scope_suffix = Self::build_scope_suffix(scope);
+                if let ScaleDataReferenceOrSignalSpec::Reference(field_ref) = field_ref {
+                    let field_name = &field_ref.field;
+                    let data_name = field_ref.data.clone();
+                    let scope_suffix = Self::build_scope_suffix(scope);
 
-                let new_data_name = format!(
-                    "{}_{}_domain_{}{}_{}",
-                    data_name, scale.name, field_name, scope_suffix, field_index
-                );
+                    let new_data_name = format!(
+                        "{}_{}_domain_{}{}_{}",
+                        data_name, scale.name, field_name, scope_suffix, field_index
+                    );
 
-                let new_data = Self::make_discrete_domain_data(
-                    &data_name,
-                    &new_data_name,
-                    field_name,
-                    sort_field.clone(),
-                    sort_op.clone(),
-                )?;
-                new_datasets.push(new_data);
+                    let new_data = Self::make_discrete_domain_data(
+                        &data_name,
+                        &new_data_name,
+                        field_name,
+                        sort_field.clone(),
+                        sort_op.clone(),
+                    )?;
+                    new_datasets.push(new_data);
 
-                // Compute new domain field
-                let mut new_field_ref = field_ref.clone();
-                new_field_ref.data = new_data_name.clone();
-                new_fields.push(new_field_ref);
+                    // Compute new domain field
+                    let mut new_field_ref = field_ref.clone();
+                    new_field_ref.data = new_data_name.clone();
+                    new_fields.push(new_field_ref);
 
-                // Compute scope for the original referenced dataset
-                let resolved = self
-                    .task_scope
-                    .resolve_scope(&Variable::new_data(data_name.as_str()), scope)?;
-                new_dataset_scope.push(resolved.scope);
+                    // Compute scope for the original referenced dataset
+                    let resolved = self
+                        .task_scope
+                        .resolve_scope(&Variable::new_data(data_name.as_str()), scope)?;
+                    new_dataset_scope.push(resolved.scope);
+                }
             }
 
             // Create new domain specification that uses the new fields
@@ -133,8 +135,11 @@ impl<'a> SplitScaleDomainVisitor<'a> {
                 sort => sort.clone(),
             };
 
-            let new_domain = ScaleDomainSpec::FieldsReference(ScaleDataReferencesSpec {
-                fields: new_fields,
+            let new_domain = ScaleDomainSpec::FieldsReferences(ScaleFieldsReferencesSpec {
+                fields: new_fields
+                    .into_iter()
+                    .map(|f| ScaleDataReferenceOrSignalSpec::Reference(f))
+                    .collect(),
                 sort,
                 extra: Default::default(),
             });
@@ -160,7 +165,7 @@ impl<'a> SplitScaleDomainVisitor<'a> {
         &mut self,
         scale: &mut ScaleSpec,
         scope: &[u32],
-        field_ref: &ScaleDataReferenceSpec,
+        field_ref: &ScaleFieldReferenceSpec,
     ) -> Result<()> {
         let discrete_scale = scale.type_.clone().unwrap_or_default().is_discrete();
         let data_name = field_ref.data.clone();
@@ -212,7 +217,7 @@ impl<'a> SplitScaleDomainVisitor<'a> {
                 sort => sort.clone(),
             };
 
-            let new_domain = ScaleDomainSpec::FieldReference(ScaleDataReferenceSpec {
+            let new_domain = ScaleDomainSpec::FieldReference(ScaleFieldReferenceSpec {
                 data: new_data_name,
                 field: field_name.clone(),
                 sort,
@@ -243,8 +248,7 @@ impl<'a> SplitScaleDomainVisitor<'a> {
                         }
                     ]
                 }
-            ))
-            .unwrap();
+            ))?;
 
             // Create new domain specification that uses the new dataset
             let new_domain: ScaleDomainSpec = serde_json::from_value(serde_json::json!([
@@ -262,8 +266,7 @@ impl<'a> SplitScaleDomainVisitor<'a> {
                             escape_field(&new_data_name)
                         )
                 }
-            ]))
-            .unwrap();
+            ]))?;
 
             (new_data, new_domain)
         } else {
@@ -319,8 +322,7 @@ impl<'a> SplitScaleDomainVisitor<'a> {
                         }
                     ]
                }
-            ))
-            .unwrap()
+            ))?
         } else {
             // Will sort by the grouped field values
             serde_json::from_value(serde_json::json!(
@@ -341,8 +343,7 @@ impl<'a> SplitScaleDomainVisitor<'a> {
                         }
                     ]
                }
-            ))
-            .unwrap()
+            ))?
         })
     }
 }
