@@ -190,15 +190,22 @@ impl VegaFusionRuntime {
         request: PreTransformSpecRequest,
     ) -> Result<PreTransformSpecResult> {
         // Extract options
-        let (row_limit, preserve_interactivity, inline_pretransform_datasets) =
+        let (row_limit, preserve_interactivity, inline_pretransform_datasets, keep_variables) =
             if let Some(opts) = request.opts {
+                // Convert keep_variables to ScopedVariable
+                let keep_variables: Vec<ScopedVariable> = opts
+                    .keep_variables
+                    .into_iter()
+                    .map(|var| (var.variable.unwrap(), var.scope))
+                    .collect();
                 (
                     opts.row_limit,
                     opts.preserve_interactivity,
                     opts.inline_datasets,
+                    keep_variables,
                 )
             } else {
-                (None, true, Default::default())
+                (None, true, Default::default(), Default::default())
             };
 
         let inline_datasets = inline_pretransform_datasets
@@ -222,6 +229,7 @@ impl VegaFusionRuntime {
                 row_limit,
                 preserve_interactivity,
                 inline_datasets,
+                keep_variables,
             )
             .await?;
 
@@ -230,7 +238,7 @@ impl VegaFusionRuntime {
             result: Some(pre_transform_spec_result::Result::Response(
                 PreTransformSpecResponse {
                     spec: serde_json::to_string(&transformed_spec)
-                        .expect("Failed to convert chart spec to string"),
+                        .with_context(|| "Failed to convert chart spec to string")?,
                     warnings,
                 },
             )),
@@ -239,6 +247,7 @@ impl VegaFusionRuntime {
         Ok(response)
     }
 
+    #[allow(clippy::too_many_arguments)]
     pub async fn pre_transform_spec(
         &self,
         spec: &ChartSpec,
@@ -247,6 +256,7 @@ impl VegaFusionRuntime {
         row_limit: Option<u32>,
         preserve_interactivity: bool,
         inline_datasets: HashMap<String, VegaFusionDataset>,
+        keep_variables: Vec<ScopedVariable>,
     ) -> Result<(ChartSpec, Vec<PreTransformSpecWarning>)> {
         let input_spec = spec;
 
@@ -257,6 +267,7 @@ impl VegaFusionRuntime {
                 default_input_tz,
                 preserve_interactivity,
                 inline_datasets,
+                keep_variables,
             )
             .await?;
 
@@ -505,8 +516,7 @@ impl VegaFusionRuntime {
         let task_scope = plan.server_spec.to_task_scope().unwrap();
         let tasks = plan
             .server_spec
-            .to_tasks(&tz_config, &dataset_fingerprints)
-            .unwrap();
+            .to_tasks(&tz_config, &dataset_fingerprints)?;
         let task_graph = TaskGraph::new(tasks, &task_scope).unwrap();
         let task_graph_mapping = task_graph.build_mapping();
 
@@ -586,6 +596,12 @@ impl VegaFusionRuntime {
         let local_tz = request.local_tz;
         let default_input_tz = request.default_input_tz;
         let preserve_interactivity = request.preserve_interactivity;
+        let extract_threshold = request.extract_threshold;
+        let keep_variables: Vec<ScopedVariable> = request
+            .keep_variables
+            .into_iter()
+            .map(|var| (var.variable.unwrap(), var.scope))
+            .collect();
 
         let (spec, datasets, warnings) = self
             .pre_transform_extract(
@@ -593,7 +609,9 @@ impl VegaFusionRuntime {
                 &local_tz,
                 &default_input_tz,
                 preserve_interactivity,
+                extract_threshold as usize,
                 inline_datasets,
+                keep_variables,
             )
             .await?;
 
@@ -623,13 +641,16 @@ impl VegaFusionRuntime {
         Ok(result)
     }
 
+    #[allow(clippy::too_many_arguments)]
     pub async fn pre_transform_extract(
         &self,
         spec: &ChartSpec,
         local_tz: &str,
         default_input_tz: &Option<String>,
         preserve_interactivity: bool,
+        extract_threshold: usize,
         inline_datasets: HashMap<String, VegaFusionDataset>,
+        keep_variables: Vec<ScopedVariable>,
     ) -> Result<(
         ChartSpec,
         Vec<PreTransformExtractDataset>,
@@ -644,6 +665,7 @@ impl VegaFusionRuntime {
                 default_input_tz,
                 preserve_interactivity,
                 inline_datasets,
+                keep_variables,
             )
             .await?;
 
@@ -682,7 +704,7 @@ impl VegaFusionRuntime {
                         // Set inline value
                         data.values = Some(input_values);
                     } else if let TaskValue::Table(table) = export_update.value {
-                        if table.num_rows() <= 20 {
+                        if table.num_rows() <= extract_threshold {
                             // Inline small tables
                             data.values = Some(table.to_json()?);
                         } else {
@@ -725,6 +747,7 @@ impl VegaFusionRuntime {
         default_input_tz: &Option<String>,
         preserve_interactivity: bool,
         inline_datasets: HashMap<String, VegaFusionDataset>,
+        keep_variables: Vec<ScopedVariable>,
     ) -> Result<(SpecPlan, Vec<ExportUpdateArrow>)> {
         // Create spec plan
         let plan = SpecPlan::try_new(
@@ -733,6 +756,7 @@ impl VegaFusionRuntime {
                 stringify_local_datetimes: true,
                 extract_inline_data: true,
                 allow_client_to_server_comms: !preserve_interactivity,
+                keep_variables,
                 ..Default::default()
             },
         )?;
@@ -764,7 +788,7 @@ impl VegaFusionRuntime {
         for var in &plan.comm_plan.server_to_client {
             let node_index = task_graph_mapping
                 .get(var)
-                .unwrap_or_else(|| panic!("Failed to lookup variable '{var:?}'"));
+                .with_context(|| format!("Failed to lookup variable '{var:?}'"))?;
             let value = self
                 .get_node_value(
                     Arc::new(task_graph.clone()),
@@ -772,7 +796,7 @@ impl VegaFusionRuntime {
                     inline_datasets.clone(),
                 )
                 .await
-                .expect("Failed to get node value");
+                .with_context(|| "Failed to get node value")?;
 
             init.push(ExportUpdateArrow {
                 namespace: ExportUpdateNamespace::try_from(var.0.namespace()).unwrap(),
