@@ -1,5 +1,6 @@
 use crate::compile::data_type::ToSqlDataType;
 use crate::compile::scalar::ToSqlScalar;
+use arrow::datatypes::DataType;
 use datafusion_common::{DFSchema, ScalarValue};
 use sqlparser::ast::{
     BinaryOperator as SqlBinaryOperator, Expr as SqlExpr, Function as SqlFunction,
@@ -17,7 +18,7 @@ use datafusion_expr::{
 
 use crate::compile::function_arg::ToSqlFunctionArg;
 use crate::compile::order::ToSqlOrderByExpr;
-use crate::dialect::{Dialect, UnorderedRowNumberMode};
+use crate::dialect::{Dialect, TryCastMode, UnorderedRowNumberMode};
 use vegafusion_common::data::scalar::ScalarValueHelpers;
 use vegafusion_common::error::{Result, VegaFusionError};
 
@@ -28,7 +29,7 @@ pub trait ToSqlExpr {
 impl ToSqlExpr for Expr {
     fn to_sql(&self, dialect: &Dialect, schema: &DFSchema) -> Result<SqlExpr> {
         match self {
-            Expr::Alias(_, _) => {
+            Expr::Alias(_) => {
                 // Alias expressions need to be handled at a higher level
                 Err(VegaFusionError::internal(format!(
                     "Alias cannot be converted to SQL: {self:?}"
@@ -196,10 +197,60 @@ impl ToSqlExpr for Expr {
                 })
             }
             Expr::TryCast(expr::TryCast { expr, data_type }) => {
-                let data_type = data_type.to_sql(dialect)?;
-                Ok(SqlExpr::TryCast {
-                    expr: Box::new(expr.to_sql(dialect, schema)?),
-                    data_type,
+                let from_dtype = expr.get_type(schema)?;
+                let sql_data_type = data_type.to_sql(dialect)?;
+                let cast_expr = if let Some(transformer) = dialect
+                    .cast_transformers
+                    .get(&(from_dtype.clone(), data_type.clone()))
+                {
+                    // Cast transformer overrides TryCast as well as Cast
+                    transformer.transform(expr.as_ref(), dialect, schema)?
+                } else {
+                    match &dialect.try_cast_mode {
+                        TryCastMode::Supported => SqlExpr::TryCast {
+                            expr: Box::new(expr.to_sql(dialect, schema)?),
+                            data_type: sql_data_type,
+                        },
+                        TryCastMode::JustUseCast => SqlExpr::Cast {
+                            expr: Box::new(expr.to_sql(dialect, schema)?),
+                            data_type: sql_data_type,
+                        },
+                        TryCastMode::SafeCast => SqlExpr::SafeCast {
+                            expr: Box::new(expr.to_sql(dialect, schema)?),
+                            data_type: sql_data_type,
+                        },
+                        TryCastMode::SupportedOnStringsOtherwiseJustCast => {
+                            if let DataType::Utf8 | DataType::LargeUtf8 = from_dtype {
+                                // TRY_CAST is supported
+                                SqlExpr::TryCast {
+                                    expr: Box::new(expr.to_sql(dialect, schema)?),
+                                    data_type: sql_data_type,
+                                }
+                            } else {
+                                // Fall back to regular CAST
+                                SqlExpr::Cast {
+                                    expr: Box::new(expr.to_sql(dialect, schema)?),
+                                    data_type: sql_data_type,
+                                }
+                            }
+                        }
+                    }
+                };
+
+                // Handle manual null propagation
+                Ok(if dialect.cast_propagates_null {
+                    cast_expr
+                } else {
+                    // Need to manually propagate nulls through cast
+                    let condition = Expr::IsNotNull(expr.clone()).to_sql(dialect, schema)?;
+                    let result = cast_expr;
+                    let else_result = lit(ScalarValue::Null).to_sql(dialect, schema)?;
+                    SqlExpr::Case {
+                        operand: None,
+                        conditions: vec![condition],
+                        results: vec![result],
+                        else_result: Some(Box::new(else_result)),
+                    }
                 })
             }
             Expr::Sort { .. } => {
@@ -297,7 +348,6 @@ impl ToSqlExpr for Expr {
                     BuiltinScalarFunction::ArrayAppend => "array_append",
                     BuiltinScalarFunction::ArrayConcat => "array_concat",
                     BuiltinScalarFunction::ArrayDims => "array_dims",
-                    BuiltinScalarFunction::ArrayFill => "array_fill",
                     BuiltinScalarFunction::ArrayLength => "array_length",
                     BuiltinScalarFunction::ArrayNdims => "array_ndims",
                     BuiltinScalarFunction::ArrayPosition => "array_position",
@@ -307,6 +357,11 @@ impl ToSqlExpr for Expr {
                     BuiltinScalarFunction::ArrayReplace => "array_replace",
                     BuiltinScalarFunction::ArrayToString => "array_to_string",
                     BuiltinScalarFunction::Cardinality => "array_cardinality",
+                    BuiltinScalarFunction::Decode => "decode",
+                    BuiltinScalarFunction::Encode => "encode",
+                    BuiltinScalarFunction::Cot => "cot",
+                    BuiltinScalarFunction::ArrayContains => "array_contains",
+                    BuiltinScalarFunction::ArrayFill => "array_fill",
                     BuiltinScalarFunction::TrimArray => "trim_array",
                 };
                 translate_scalar_function(fun_name, args, dialect, schema)
@@ -522,9 +577,6 @@ impl ToSqlExpr for Expr {
                 "GroupingSet cannot be converted to SQL",
             )),
             Expr::Like { .. } => Err(VegaFusionError::internal("Like cannot be converted to SQL")),
-            Expr::ILike { .. } => Err(VegaFusionError::internal(
-                "ILike cannot be converted to SQL",
-            )),
             Expr::SimilarTo { .. } => Err(VegaFusionError::internal(
                 "SimilarTo cannot be converted to SQL",
             )),
